@@ -4,13 +4,12 @@
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
- * Software Foundation; either version 3 of the License, or (at your option) any
- * later version. See http://www.gnu.org/copyleft/lgpl.html the full text of the
- * license.
+ * Software Foundation; either version 2 or version 3 of the License.
+ * See http://www.gnu.org/copyleft/lgpl.html the full text of the license.
  */
 
-#ifndef _LIGHTDM_HOSTNAME_H_
-#define _LIGHTDM_HOSTNAME_H_
+#ifndef LIGHTDM_HOSTNAME_H_
+#define LIGHTDM_HOSTNAME_H_
 
 #include <glib-object.h>
 
@@ -20,4 +19,4 @@ const gchar *lightdm_get_hostname (void);
 
 G_END_DECLS
 
-#endif /* _LIGHTDM_HOSTNAME_H_ */
+#endif /* LIGHTDM_HOSTNAME_H_ */
