@@ -1,3 +1,5 @@
+#include <config.h>
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -12,12 +14,16 @@
 #include <glib.h>
 #include <security/pam_appl.h>
 #include <utmpx.h>
+#include <sys/mman.h>
 
+#include "configuration.h"
 #include "session-child.h"
 #include "session.h"
 #include "console-kit.h"
+#include "login1.h"
 #include "privileges.h"
-#include "xauthority.h"
+#include "x-authority.h"
+#include "configuration.h"
 
 /* Child process being run */
 static GPid child_pid = 0;
@@ -65,7 +71,7 @@ read_data (void *buf, size_t count)
 }
 
 static gchar *
-read_string ()
+read_string_full (void* (*alloc_fn)(size_t n))
 {
     int length;
     char *value;
@@ -80,13 +86,19 @@ read_string ()
         return NULL;
     }
   
-    value = g_malloc (sizeof (char) * (length + 1));
+    value = (*alloc_fn) (sizeof (char) * (length + 1));
     read_data (value, length);
     value[length] = '\0';      
 
     return value;
 }
 
+static gchar *
+read_string (void)
+{
+    return read_string_full (g_malloc);
+}
+
 static int
 pam_conv_cb (int msg_length, const struct pam_message **msg, struct pam_response **resp, void *app_data)
 {
@@ -137,7 +149,9 @@ pam_conv_cb (int msg_length, const struct pam_message **msg, struct pam_response
     for (i = 0; i < msg_length; i++)
     {
         struct pam_response *r = &response[i];
-        r->resp = read_string ();
+        // callers of this function inside pam will expect to be able to call
+        // free() on the strings we give back.  So alloc with malloc.
+        r->resp = read_string_full (malloc);
         read_data (&r->resp_retcode, sizeof (r->resp_retcode));
     }
 
@@ -155,6 +169,33 @@ signal_cb (int signum)
         exit (EXIT_SUCCESS);
 }
 
+static XAuthority *
+read_xauth (void)
+{
+    gchar *x_authority_name;
+    guint16 x_authority_family;
+    guint8 *x_authority_address;
+    gsize x_authority_address_length;
+    gchar *x_authority_number;
+    guint8 *x_authority_data;
+    gsize x_authority_data_length;
+
+    x_authority_name = read_string ();
+    if (!x_authority_name)
+        return NULL;
+
+    read_data (&x_authority_family, sizeof (x_authority_family));
+    read_data (&x_authority_address_length, sizeof (x_authority_address_length));
+    x_authority_address = g_malloc (x_authority_address_length);
+    read_data (x_authority_address, x_authority_address_length);
+    x_authority_number = read_string ();
+    read_data (&x_authority_data_length, sizeof (x_authority_data_length));
+    x_authority_data = g_malloc (x_authority_data_length);
+    read_data (x_authority_data, x_authority_data_length);
+
+    return x_authority_new (x_authority_family, x_authority_address, x_authority_address_length, x_authority_number, x_authority_name, x_authority_data, x_authority_data_length);
+}
+
 int
 session_child_run (int argc, char **argv)
 {
@@ -172,19 +213,40 @@ session_child_run (int argc, char **argv)
     gchar *authentication_result_string;
     gchar *service;
     gchar *username;
-    gchar *class;
     gchar *tty;
     gchar *remote_host_name;
     gchar *xdisplay;
-    gchar *xauth_name;
-    XAuthority *xauthority = NULL;
-    gchar *xauth_filename;
+    XAuthority *x_authority = NULL;
+    gchar *x_authority_filename;
     GDBusConnection *bus;
-    gchar *console_kit_cookie;
+    gchar *console_kit_cookie = NULL;
+    gchar *login1_session = NULL;
+
     const gchar *path;
     GError *error = NULL;
+    const gchar *locale_value;
+    gchar *locale_var;
+    static const gchar * const locale_var_names[] = {
+        "LC_COLLATE",
+        "LC_CTYPE",
+        "LC_MONETARY",
+        "LC_NUMERIC",
+        "LC_TIME",
+        "LC_MESSAGES",
+        "LC_ALL",
+        "LANG",
+        NULL
+    };
 
+#if !defined(GLIB_VERSION_2_36)
     g_type_init ();
+#endif
+
+    if (config_get_boolean (config_get_instance (), "LightDM", "lock-memory"))
+    {
+        /* Protect memory from being paged to disk, as we deal with passwords */
+        mlockall (MCL_CURRENT | MCL_FUTURE);
+    }
 
     /* Make input non-blocking */
     fd = open ("/dev/null", O_RDONLY);
@@ -221,31 +283,11 @@ session_child_run (int argc, char **argv)
     username = read_string ();
     read_data (&do_authenticate, sizeof (do_authenticate));
     read_data (&is_interactive, sizeof (is_interactive));
-    class = read_string ();
+    read_string (); /* Used to be class, now we just use the environment variable */
     tty = read_string ();
     remote_host_name = read_string ();
     xdisplay = read_string ();
-    xauth_name = read_string ();
-    if (xauth_name)
-    {
-        guint16 xauth_family;
-        guint8 *xauth_address;
-        gsize xauth_address_length;
-        gchar *xauth_number;
-        guint8 *xauth_data;
-        gsize xauth_data_length;
-
-        read_data (&xauth_family, sizeof (xauth_family));
-        read_data (&xauth_address_length, sizeof (xauth_address_length));
-        xauth_address = g_malloc (xauth_address_length);
-        read_data (xauth_address, xauth_address_length);
-        xauth_number = read_string ();
-        read_data (&xauth_data_length, sizeof (xauth_data_length));
-        xauth_data = g_malloc (xauth_data_length);
-        read_data (xauth_data, xauth_data_length);
-
-        xauthority = xauth_new (xauth_family, xauth_address, xauth_address_length, xauth_number, xauth_name, xauth_data, xauth_data_length);
-    }
+    x_authority = read_xauth ();
 
     /* Setup PAM */
     result = pam_start (service, username, &conversation, &pam_handle);
@@ -265,14 +307,14 @@ session_child_run (int argc, char **argv)
         pam_set_item (pam_handle, PAM_TTY, tty);    
 
 #ifdef PAM_XAUTHDATA
-    if (xauthority)
+    if (x_authority)
     {
         struct pam_xauth_data value;
 
-        value.name = (char *) xauth_get_authorization_name (xauthority);
-        value.namelen = strlen (xauth_get_authorization_name (xauthority));
-        value.data = (char *) xauth_get_authorization_data (xauthority);
-        value.datalen = xauth_get_authorization_data_length (xauthority);
+        value.name = (char *) x_authority_get_authorization_name (x_authority);
+        value.namelen = strlen (x_authority_get_authorization_name (x_authority));
+        value.data = (char *) x_authority_get_authorization_data (x_authority);
+        value.datalen = x_authority_get_authorization_data_length (x_authority);
         pam_set_item (pam_handle, PAM_XAUTHDATA, &value);
     }
 #endif
@@ -317,6 +359,17 @@ session_child_run (int argc, char **argv)
             pam_putenv (pam_handle, g_strdup_printf ("LOGNAME=%s", username));
             pam_putenv (pam_handle, g_strdup_printf ("HOME=%s", user_get_home_directory (user)));
             pam_putenv (pam_handle, g_strdup_printf ("SHELL=%s", user_get_shell (user)));
+
+            /* Let the greeter and user session inherit the system default locale */
+            for (i = 0; locale_var_names[i] != NULL; i++)
+            {
+                if ((locale_value = g_getenv (locale_var_names[i])) != NULL)
+                {
+                    locale_var = g_strdup_printf ("%s=%s", locale_var_names[i], locale_value);
+                    pam_putenv (pam_handle, locale_var);
+                    g_free (locale_var);
+                }
+            }
         }
     }
 
@@ -341,7 +394,20 @@ session_child_run (int argc, char **argv)
 
     /* Get the command to run (blocks) */
     log_filename = read_string ();
-    xauth_filename = read_string ();
+    if (version >= 1)
+    {
+        g_free (tty);
+        tty = read_string ();      
+    }
+    x_authority_filename = read_string ();
+    if (version >= 1)
+    {
+        g_free (xdisplay);
+        xdisplay = read_string ();
+        if (x_authority)
+            g_object_unref (x_authority);
+        x_authority = read_xauth ();
+    }
     read_data (&env_length, sizeof (env_length));
     for (i = 0; i < env_length; i++)
         pam_putenv (pam_handle, read_string ());
@@ -401,60 +467,65 @@ session_child_run (int argc, char **argv)
     if (!bus)
         return EXIT_FAILURE;
 
-    /* Open a Console Kit session */
-    g_variant_builder_init (&ck_parameters, G_VARIANT_TYPE ("(a(sv))"));
-    g_variant_builder_open (&ck_parameters, G_VARIANT_TYPE ("a(sv)"));
-    g_variant_builder_add (&ck_parameters, "(sv)", "unix-user", g_variant_new_int32 (user_get_uid (user)));
-    if (g_strcmp0 (class, XDG_SESSION_CLASS_GREETER) == 0)
-        g_variant_builder_add (&ck_parameters, "(sv)", "session-type", g_variant_new_string ("LoginWindow"));
-    if (xdisplay)
+    if (login1_is_running ())
     {
-        g_variant_builder_add (&ck_parameters, "(sv)", "x11-display", g_variant_new_string (xdisplay));
-        if (tty)
-            g_variant_builder_add (&ck_parameters, "(sv)", "x11-display-device", g_variant_new_string (tty));
+        login1_session = login1_get_session_id ();
+        write_string (login1_session);
     }
-    if (remote_host_name)
+
+    if (!login1_session)
     {
-        g_variant_builder_add (&ck_parameters, "(sv)", "is-local", g_variant_new_boolean (FALSE));
-        g_variant_builder_add (&ck_parameters, "(sv)", "remote-host-name", g_variant_new_string (remote_host_name));
-    }
-    else
-        g_variant_builder_add (&ck_parameters, "(sv)", "is-local", g_variant_new_boolean (TRUE));
-    console_kit_cookie = ck_open_session (&ck_parameters);
-    write_string (console_kit_cookie);
-    if (console_kit_cookie)
-    {
-        gchar *value;
-        value = g_strdup_printf ("XDG_SESSION_COOKIE=%s", console_kit_cookie);
-        pam_putenv (pam_handle, value);
-        g_free (value);
+        /* Open a Console Kit session */
+        g_variant_builder_init (&ck_parameters, G_VARIANT_TYPE ("(a(sv))"));
+        g_variant_builder_open (&ck_parameters, G_VARIANT_TYPE ("a(sv)"));
+        g_variant_builder_add (&ck_parameters, "(sv)", "unix-user", g_variant_new_int32 (user_get_uid (user)));
+        if (g_strcmp0 (pam_getenv (pam_handle, "XDG_SESSION_CLASS"), "greeter") == 0)
+            g_variant_builder_add (&ck_parameters, "(sv)", "session-type", g_variant_new_string ("LoginWindow"));
+        if (xdisplay)
+        {
+            g_variant_builder_add (&ck_parameters, "(sv)", "x11-display", g_variant_new_string (xdisplay));
+            if (tty)
+                g_variant_builder_add (&ck_parameters, "(sv)", "x11-display-device", g_variant_new_string (tty));
+        }
+        if (remote_host_name)
+        {
+            g_variant_builder_add (&ck_parameters, "(sv)", "is-local", g_variant_new_boolean (FALSE));
+            g_variant_builder_add (&ck_parameters, "(sv)", "remote-host-name", g_variant_new_string (remote_host_name));
+        }
+        else
+            g_variant_builder_add (&ck_parameters, "(sv)", "is-local", g_variant_new_boolean (TRUE));
+        console_kit_cookie = ck_open_session (&ck_parameters);
+        write_string (console_kit_cookie);
+        if (console_kit_cookie)
+        {
+            gchar *value;
+            value = g_strdup_printf ("XDG_SESSION_COOKIE=%s", console_kit_cookie);
+            pam_putenv (pam_handle, value);
+            g_free (value);
+        }
     }
 
     /* Write X authority */
-    if (xauthority)
+    if (x_authority)
     {
-        GFile *file;
         gboolean drop_privileges, result;
         gchar *value;
         GError *error = NULL;
 
-        file = g_file_new_for_path (xauth_filename);
-
         drop_privileges = geteuid () == 0;
         if (drop_privileges)
             privileges_drop (user);
-        result = xauth_write (xauthority, XAUTH_WRITE_MODE_REPLACE, file, &error);
+        result = x_authority_write (x_authority, XAUTH_WRITE_MODE_REPLACE, x_authority_filename, &error);
         if (drop_privileges)
             privileges_reclaim ();
 
-        g_object_unref (file);
         if (error)
             g_printerr ("Error writing X authority: %s\n", error->message);
         g_clear_error (&error);
         if (!result)
             return EXIT_FAILURE;
 
-        value = g_strdup_printf ("XAUTHORITY=%s", xauth_filename);
+        value = g_strdup_printf ("XAUTHORITY=%s", x_authority_filename);
         pam_putenv (pam_handle, value);
         g_free (value);
     }
@@ -529,7 +600,7 @@ session_child_run (int argc, char **argv)
     if (child_pid > 0)
     {
         /* Log to utmp */
-        if (g_strcmp0 (class, XDG_SESSION_CLASS_GREETER) != 0)
+        if (g_strcmp0 (pam_getenv (pam_handle, "XDG_SESSION_CLASS"), "greeter") != 0)
         {
             struct utmpx ut;
             struct timeval tv;
@@ -537,8 +608,10 @@ session_child_run (int argc, char **argv)
             memset (&ut, 0, sizeof (ut));
             ut.ut_type = USER_PROCESS;
             ut.ut_pid = child_pid;
-            strncpy (ut.ut_line, tty + strlen ("/dev/"), sizeof (ut.ut_line));
-            strncpy (ut.ut_id, xdisplay, sizeof (ut.ut_id));
+            if (tty)
+                strncpy (ut.ut_line, tty + strlen ("/dev/"), sizeof (ut.ut_line));
+            if (xdisplay)
+                strncpy (ut.ut_id, xdisplay, sizeof (ut.ut_id));
             strncpy (ut.ut_user, username, sizeof (ut.ut_user));
             if (xdisplay)
                 strncpy (ut.ut_host, xdisplay, sizeof (ut.ut_host));
@@ -558,7 +631,7 @@ session_child_run (int argc, char **argv)
         child_pid = 0;
 
         /* Log to utmp */
-        if (g_strcmp0 (class, XDG_SESSION_CLASS_GREETER) != 0)
+        if (g_strcmp0 (pam_getenv (pam_handle, "XDG_SESSION_CLASS"), "greeter") != 0)
         {
             struct utmpx ut;
             struct timeval tv;
@@ -566,8 +639,10 @@ session_child_run (int argc, char **argv)
             memset (&ut, 0, sizeof (ut));
             ut.ut_type = DEAD_PROCESS;
             ut.ut_pid = child_pid;
-            strncpy (ut.ut_line, tty + strlen ("/dev/"), sizeof (ut.ut_line));
-            strncpy (ut.ut_id, xdisplay, sizeof (ut.ut_id));
+            if (tty)
+                strncpy (ut.ut_line, tty + strlen ("/dev/"), sizeof (ut.ut_line));
+            if (xdisplay)
+                strncpy (ut.ut_id, xdisplay, sizeof (ut.ut_id));
             strncpy (ut.ut_user, username, sizeof (ut.ut_user));
             if (xdisplay)
                 strncpy (ut.ut_host, xdisplay, sizeof (ut.ut_host));
@@ -585,22 +660,18 @@ session_child_run (int argc, char **argv)
     }
 
     /* Remove X authority */
-    if (xauthority)
+    if (x_authority)
     {
-        GFile *file;
         gboolean drop_privileges, result;
         GError *error = NULL;
 
-        file = g_file_new_for_path (xauth_filename);
-
         drop_privileges = geteuid () == 0;
         if (drop_privileges)
             privileges_drop (user);
-        result = xauth_write (xauthority, XAUTH_WRITE_MODE_REMOVE, file, &error);
+        result = x_authority_write (x_authority, XAUTH_WRITE_MODE_REMOVE, x_authority_filename, &error);
         if (drop_privileges)
             privileges_reclaim ();
 
-        g_object_unref (file);
         if (error)
             g_printerr ("Error removing X authority: %s\n", error->message);
         g_clear_error (&error);
